﻿using BingPaper.Interfaces;
using BingPaper.PInvoke;
using BingPaper.Util;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;

namespace BingPaper.Impl.Command
{
    class SetWindowColorCommand : ICommand
    {
        public void Run(ICommandContext context)
        {
            if (context == null)
                throw new ArgumentNullException("context");
            if (context.Config.AutoWindowColor)
                SetWindowColor(context.ImageSavePath, context.Logger);
        }

        private void SetWindowColor(string path, ILogger logger)
        {
            if (!IsSupportWindowColor())
            {
                logger.LogWarn("Current system not support window color.");
                return;
            }
            bool compositeEnable;

            if (0 == DWMWrapper.DwmIsCompositionEnabled(out compositeEnable) && compositeEnable)
            {
                COLORIZATIONPARAMS colrParam;
                int hr;
                if (0 == (hr = DWMWrapper.DwmGetColorizationParameters(out colrParam)))
                {
                    logger.LogMessage("Calculating window color");
                    Color calculatedColor;
                    using (var image = Image.FromFile(path))
                    {
                        calculatedColor = WindowColorCalculator.Calculate(image);
                    }
                    colrParam.clrColor.R = calculatedColor.R;
                    colrParam.clrColor.G = calculatedColor.G;
                    colrParam.clrColor.B = calculatedColor.B;
                    logger.LogMessage(string.Format("Set window color: {0}", calculatedColor));
                    if (0 != (hr = DWMWrapper.DwmSetColorizationParameters(ref colrParam, 0)))
                        logger.LogError(string.Format("DwmSetColorizationParameters failed : 0x{0:X8}", hr));
                }
                else
                    logger.LogError(string.Format("DwmGetColorizationParameters failed : 0x{0:X8}", hr));
            }
            else
            {
                logger.LogWarn("Not set window color : Desktop Window Manager (DWM) composition is not enabled");
                return;
            }
        }

        private bool IsSupportWindowColor()
        {
            var os = Environment.OSVersion;
            if (os.Platform != PlatformID.Win32NT)
                return false;
            if (os.Version.Major < 6) // vista is 6
                return false;
            return true;
        }
    }
}
