using LitJson;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Net;
namespace BingPaper.Impl
{
	internal class ImageDownloader
	{
		private const string LIST_URL = "/HPImageArchive.aspx?format=js&idx=0&n=1&setmkt={0}";
		
		private const string HD_IMAGE_URL = "/hpwp/{0}?FORM=HDRHME&setmkt={1}";
		
		private static readonly Uri BASE_URL = new Uri("http://www.bing.com");
		
		private ILogger _logger;
        private string _savedPath;
		
		public ImageDownloader(ILogger logger, string savedPath)
		{
			this._logger = logger;
            _savedPath = savedPath;
		}
		
		public string DownloadImage(string prevImageName, string countryCode)
		{
			var imageInfo = this.GetImageName(countryCode);
			this._logger.LogMessage("Bing image name is " + imageInfo.Key);
			if (string.Equals(imageInfo.Key, prevImageName, StringComparison.OrdinalIgnoreCase))
			{
				this._logger.LogMessage("Bing image not updated.");
				return string.Empty;
			}
            this.DownloadImage(imageInfo.Key, imageInfo.Value, countryCode);
			return imageInfo.Key;
		}
		
		private KeyValuePair<String, String> GetImageName(string countryCode)
		{
			this._logger.LogMessage("Get Bing image name...");
			Uri requestUri = new Uri(ImageDownloader.BASE_URL, string.Format(LIST_URL, countryCode));
			WebRequest webRequest = WebRequest.Create(requestUri);
			webRequest.Proxy = WebRequest.GetSystemWebProxy();
			webRequest.Proxy.Credentials = System.Net.CredentialCache.DefaultCredentials;
			WebResponse response = webRequest.GetResponse();
			var result = new KeyValuePair<string, string>(string.Empty, string.Empty);
			using (TextReader textReader = new StreamReader(response.GetResponseStream()))
			{
				JsonData jsonData = JsonMapper.ToObject(textReader.ReadToEnd());
				JsonData jsonData2 = jsonData["images"];
				if (jsonData2.Count > 0)
				{
					result = new KeyValuePair<string, string>(jsonData2[0]["url"].ToString(), jsonData2[0]["hsh"].ToString());
				}
			}
			return result;
		}
		
		private void DownloadImage(string lowQulityPath, string imageHsh, string countryCode)
		{
			try{
				DownloadHDImage(imageHsh, countryCode);
			}catch(Exception err){
				_logger.LogError(string.Format("Download HD Image failed : {0}", err.Message));
				_logger.LogMessage("Download Low Image");
				DownloadLDImage(lowQulityPath);
			}
		}
		
		private void DownloadHDImage(string imageHsh, string countryCode)
		{
			Uri imageUri = new Uri(BASE_URL, string.Format(HD_IMAGE_URL, imageHsh, countryCode));
			Download(imageUri);
		}
		
		private void DownloadLDImage(string lowQulityPath)
		{
			Uri imageUri = new Uri(BASE_URL, lowQulityPath);
			Download(imageUri);
		}
		
		private void Download(Uri imageUri)
		{
			this._logger.LogMessage("Download Image...");
			WebRequest webRequest = WebRequest.Create(imageUri);
			webRequest.Proxy = WebRequest.GetSystemWebProxy();
			webRequest.Proxy.Credentials = System.Net.CredentialCache.DefaultCredentials;
			WebResponse response = webRequest.GetResponse();
			using (Stream responseStream = response.GetResponseStream())
			{
                using (Image jpg = Image.FromStream(responseStream))
                {
                    jpg.Save(_savedPath, System.Drawing.Imaging.ImageFormat.Bmp);
                }
			}
            this._logger.LogMessage("Image saved at " + _savedPath);
		}
	}
}
