﻿using System;
using System.IO;
using System.Text;

using BingPaper.Interfaces;
using LitJson;

namespace BingPaper.Impl
{
	internal class JsonConfigStorage : IConfigStorage
	{
		private ILogger _logger;
		
		public JsonConfigStorage(ILogger logger)
		{
			if (logger == null)
				throw new ArgumentNullException("logger");
			this._logger = logger;
		}
		
		public Config LoadConfig()
		{
			ensureInitialized();
			using(TextReader reader = ReaderWriterProvider.createReader())
			{
				try{
					Config config = JsonMapper.ToObject<Config>(reader);
					if (config == null){
						_logger.LogWarn("empty config, use default config instead.");
						config = Config.CreateDefaultConfig();
					}
					return config;
				}catch(ApplicationException err){
					_logger.LogError(string.Format("load config failed : {0}", err.Message));
					_logger.LogMessage("created empty config");
					return Config.CreateDefaultConfig();
				}
			}
		}

		void ensureInitialized()
		{
			if (ReaderWriterProvider == null)
				throw new InvalidOperationException("ReaderWriterProvider properties is not initialized");
		}
		
		public void SaveConfig(Config config)
		{
			if (config == null)
				throw new ArgumentNullException("config");
			ensureInitialized();
			using(TextWriter writer = ReaderWriterProvider.createWriter())
			{
				try{
					writer.Write(JsonMapper.ToJson(config));
				}catch(ApplicationException err){
					_logger.LogError(string.Format("save config failed : {0}", err.Message));
				}
			}
		}
		
		public IReaderWriterProvider ReaderWriterProvider {get; set;}
	}
}
