﻿using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace BingPaper.Interfaces
{
	internal class Config
	{
		internal const int PERIOD_MINUTE = 0;
		internal const int PERIOD_TIME = 1;
		
		internal const int WP_STYLE_CENTER = 0;
		internal const int WP_STYLE_STRETCH = 1;
		internal const int WP_STYLE_FIT = 2;
		internal const int WP_STYLE_FILL = 3;
		internal const int WP_TILE = 4;
		
		private string _countryCode;
		
		public int PeriodType {get; set;}
		
		public String Period {get; set;}
		
		public bool PreventBlackWallpaper {get; set;}
		
		public string CountryCode
		{
			get {
				if (string.IsNullOrWhiteSpace(_countryCode))
					_countryCode = "en-US";
				return _countryCode;
			}
			set {_countryCode = value;}
		}
		
		public bool RandomCountryCode {get; set;}
		
		public int WallPaperStyle {get; set;}
		
		public bool AutoWindowColor {get; set;}
		
		public static Config CreateDefaultConfig()
		{
			return new Config
			{
				PeriodType = PERIOD_MINUTE,
				Period = "60", //update every hour
				PreventBlackWallpaper = true
			};
		}
		
		public IEnumerable<KeyValuePair<string, string>> Validate()
		{
			if (PeriodType < PERIOD_MINUTE || PeriodType > PERIOD_TIME)
				yield return new KeyValuePair<string, string>("PeriodType","Invalid Period Type");
			String err;
			if (PeriodType == PERIOD_MINUTE && null != (err = ValidateMinute()))
				yield return new KeyValuePair<string, string>("Period", err);
			if (PeriodType == PERIOD_TIME && null != (err = ValidateTime()))
				yield return new KeyValuePair<string, string>("Period", err);
			if (null != (err = ValidateWallPaperStyle()))
				yield return new KeyValuePair<string, string>("WallPaperStyle", err);
		}
		
		private string ValidateMinute()
		{
			int value;
			if (!int.TryParse(Period, out value) || value <= 0)
				return "Period must be a positive number for PERIOD_MINUTE";
			return null;
		}
		
		private string ValidateTime()
		{
			const string ERR = "Period must be MM:SS form for PEIROD_TIME";
			Match match = Regex.Match(Period, @"^(?<hour>\d{2}):(?<minute>\d{2})$");
			if (match.Success)
			{
				int hour = int.Parse(match.Groups["hour"].Value);
				int minute = int.Parse(match.Groups["minute"].Value);
				if (hour < 0 || hour > 23 || minute < 0 || minute > 59)
					return ERR;
				return null;
			}
			
			return ERR;
		}
		
		private string ValidateWallPaperStyle()
		{
			if (WallPaperStyle < WP_STYLE_CENTER || WallPaperStyle > WP_TILE)
				return "Invalid wallpaper style";
			return null;
		}
	}
}
