﻿
#if TEST
using System;
using System.IO;
using BingPaper.Impl;
using BingPaper.Interfaces;
using NUnit.Framework;

namespace BingPaper.Test
{
	[TestFixture]
	public class JsonConfigStorageTest
	{
		private ReaderWriterProvider _readerWriterProvider;
		private JsonConfigStorage _storage;
		
		[Test]
		public void TestLoadSaveConfig()
		{
			Config config = _storage.LoadConfig();
			Assert.IsNotNull(config);
			Assert.IsTrue(config.PreventBlackWallpaper);
			Assert.AreEqual(Config.PERIOD_MINUTE, config.PeriodType);
			Assert.AreEqual("60", config.Period);
			
			config.PreventBlackWallpaper = false;
			config.PeriodType = Config.PERIOD_TIME;
			config.Period = "12:00";
			
			_storage.SaveConfig(config);
			config = _storage.LoadConfig();
			
			Assert.IsNotNull(config);
			Assert.IsFalse(config.PreventBlackWallpaper);
			Assert.AreEqual(Config.PERIOD_TIME, config.PeriodType);
			Assert.AreEqual("12:00", config.Period);
			
		}
		
		[TestFixtureSetUp]
		public void Init()
		{
			_readerWriterProvider = new ReaderWriterProvider();
			_readerWriterProvider.ConfigPath = Path.GetFullPath(".BingPaperConfig");
			_storage = new JsonConfigStorage(new ConsoleLogger()){
				ReaderWriterProvider = _readerWriterProvider
			};
		}
		
		[TestFixtureTearDown]
		public void Dispose()
		{
			if (File.Exists(_readerWriterProvider.ConfigPath))
				File.Delete(_readerWriterProvider.ConfigPath);
		}
	}
}
#endif
