﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Windows.Forms;

using BingPaper.Impl;
using BingPaper.Interfaces;

namespace BingPaper.UI
{
	internal partial class MainForm : Form
	{
		private const int WM_SETTINGCHANGE = 0x001A;
				
		private Model Model {get; set;}
		
		private IAppCore AppCore {get; set;}
		
		private ILogger Logger {get; set;}
		
		private bool _exit = false;
				
		public MainForm(Model model, IAppCore appcore, ILogger logger)
		{
			if (model == null)
				throw new ArgumentNullException("model");
			if (appcore == null)
				throw new ArgumentNullException("appcore");
			if (logger == null)
				throw new ArgumentNullException("logger");
			
			Model = model;
			AppCore = appcore;
			Logger = logger;
			
			InitializeComponent();
			this.txtLog.DataBindings.Add("Lines", this.Model, "Logs");
			this.butUpdate.DataBindings.Add("Enabled", this.AppCore, "IsIdle");
			this.butSetting.DataBindings.Add("Enabled", this.AppCore, "IsIdle");
		}
		
		protected override void WndProc(ref Message m)
		{
			if (m.Msg == WM_SETTINGCHANGE && m.WParam.ToInt32() == (int)PInvoke.SPI.SPI_SETDESKWALLPAPER)
			{
				Logger.LogMessage("Desktop Wallpaper changed.");
				AppCore.NotifyWallpaperChanged();
			}
			base.WndProc(ref m);
		}
		
		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);
			#if RELEASE
			AppCore.Run();
			WindowState = FormWindowState.Minimized;
			#endif
		}
		
		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);
			if (base.WindowState == FormWindowState.Minimized)
			{
				this.notifyIcon.Visible = true;
				base.ShowInTaskbar = false;
				base.Hide();
				return;
			}
			this.notifyIcon.Visible = false;
		}
		
		protected override void OnFormClosing(FormClosingEventArgs e)
		{
			if (e.CloseReason == CloseReason.UserClosing && !_exit)
			{
				e.Cancel = true;
				WindowState = FormWindowState.Minimized;
			}
			base.OnFormClosing(e);
		}
		
		private void UpdateClick(object sender, EventArgs e)
		{
			AppCore.Stop();
			AppCore.Run();
		}
		
		private void ClearClick(object sender, EventArgs e)
		{
			this.Model.ClearLog();
		}
		
		private void ShowWindow(object sender, EventArgs e)
		{
			base.Show();
			base.WindowState = FormWindowState.Normal;
			base.ShowInTaskbar = true;
		}
		
		private void SettingClick(object sender, EventArgs e)
		{
			Config config = AppCore.ConfigStorage.LoadConfig();
			SettingForm dialog = new SettingForm(config);
			if (DialogResult.OK == dialog.ShowDialog(this))
			{
				AppCore.ConfigStorage.SaveConfig(dialog.Config);
				#if RELEASE
				AppCore.Stop();
				AppCore.Run();
				#endif
			}
		}
		
		private void ExitClick(object sender, EventArgs e)
		{
			_exit = true;
			this.Close();
		}
	}
}
