using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Threading;
namespace BingPaper.UI
{
	internal class Model : INotifyPropertyChanged
	{
		const int MAX_LOG = 100;
		private List<string> _logs = new List<string>();
		
		public event PropertyChangedEventHandler PropertyChanged;

		public string[] Logs
		{
			get
			{
				return this._logs.ToArray();
			}
		}
		
		public void AddLog(string msg)
		{
			this._logs.Add(msg);
			if (this._logs.Count > MAX_LOG)
				this._logs.RemoveRange(0, this._logs.Count - MAX_LOG);
			this.OnPropertyChanged(new PropertyChangedEventArgs("Logs"));
		}
		
		public void ClearLog()
		{
			this._logs.Clear();
			this.OnPropertyChanged(new PropertyChangedEventArgs("Logs"));
		}
		
		protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
		{
			PropertyChangedEventHandler propertyChangedEventHandler = Interlocked.CompareExchange<PropertyChangedEventHandler>(ref this.PropertyChanged, null, null);
			if (propertyChangedEventHandler != null)
			{
				propertyChangedEventHandler(this, e);
			}
		}
	}
}
